/*
**
**
** MODULE: manager.c
**
**
** PROGRAM: Portable PHNBK2 Application (OpenVMS,DOS,NT,OSF/1,ULTRIX)
**
**
** ABSTRACT: PHNBK2 is a sample RPC application intended to illustrate
**           the basics of RPC interoperation between DCE platforms
**           and Microsoft platforms.  PHNBK2 extends the capabilities
**           of PHNBK by using context handles, and the CDS nameservice.
**
**          
**
**
*/      

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "phnbk2.h"

#ifndef VMS
#include <malloc.h>
#endif

#ifdef WIN32 
#include "dceport.h"
#endif

typedef struct _p_context
    {
    /*
    ** Phonebook handle
    */
    FILE * filehandle;
    /*
    ** Previous phonebook operation
    */
    short previous_operation;
    } p_context;



/*
**
** FUNCTION:  getfileline
**
** PURPOSE:
**      Retrieve Lines from input file
**
*/
int 
getfileline
           (
            line, 
            phone
           )
idl_char * line;
FILE * phone;
{
    /*
    ** Each call of this routine returns a line of the 
    **    phonebook file.  On EOF, it returns -1.
    */
    char ch;

    while ((ch = fgetc(phone)) != '\n' && ch != EOF)
        {
        /*
        ** Tabs are unpredictable, so substitute
        **    three spaces if you run across a tab...
        */
        if (ch == '\t')
            {
            *line++ = ' ';
            *line++ = ' ';
            *line++ = ' ';
            }
        else        
            *line++ = ch;
        
        }

    *line++ = '\0';

    if (ch == EOF)
        return (END);
    else
        return (NORMAL);
}

/*
**
** FUNCTION:  open_phonebook_file
**
** PURPOSE:   
**      Open phonebook file and establish a context handle that
**      reflects the client's phonebook "state"
**
*/
void 
open_phonebook_file
           (
            ph
           )
phonebook_handle *ph;
{
    /*
    ** This structure is defined in phnbk2.idl, and contains both
    **   a filehandle and a short that keeps track of the previous
    **   phonebook operation.
    */
    p_context *pc_p;

    /*
    ** Allocate the required amount of memory for a p_context
    **    structure
    */ 

    pc_p = (p_context *) malloc ( sizeof (p_context) );
    /*
    ** Allocate a file handle for this client's operations
    */

    pc_p->filehandle = fopen ("phnbk.txt","r");
    /*
    ** Initialize the p_context...
    **
    ** The previous operation is 0 (no previous operation yet)
    */

    pc_p->previous_operation = 0;    
    /*
    ** Return the context handle
    */
    *ph = (phonebook_handle) pc_p;
        
    return ;
}
   
/*
** 
** FUNCTION:  phonebook_handle_rundown
**
** PURPOSE:  
**      This routine releases resources associated with the
**      context handle if the connection goes away.
**
*/
void 
phonebook_handle_rundown 
           (
            ph
           )
phonebook_handle ph;
{
    p_context *pc_p;


    pc_p = (p_context *)ph;
    /*
    ** Close the file associated with the filehandle
    */
    fclose (pc_p->filehandle);

    /*
    ** ...and free the memory associated with this p_context struct
    */
    free(pc_p);

    return;
}

/*
**
** FUNCTION:  close_phonebook_file
**
** PURPOSE:  
**      This routine does an orderly close of the phonebook file handle,
**      and frees resources associated with the context handle.
**
*/
void 
close_phonebook_file 
           (
            ph
           )
phonebook_handle *ph;
{
    /*
    ** Effect a context handle cleanup by calling on the
    **    rundown routine -- it already does what we want
    **    to do
    */
    phonebook_handle_rundown 
               (
                *ph
               );

    /*
    ** Reset the context handle to NULL
    */
    *ph = NULL;

    return;
}


/*
**
** FUNCTION:  lookup
**
** PURPOSE:
**      Lookup up entries in database
**
*/
short 
lookup 
      ( 
       phone,
       op,
       stringin, 
       stringout 
      )
phonebook_handle phone;
operations       op;
idl_char stringin[LINESIZE];
idl_char stringout[LINESIZE];
{
    idl_char buf[LINESIZE];
    p_context *pc_p;

    /*
    ** Remember that our context handle is a pointer to a
    **    p_context structure
    */
    pc_p = (p_context *)phone;

    /*
    ** Switch on requested operation
    */ 
    switch (op)
        {
        case  RESET:
            /*
            **  Reset context
            */
            printf("Phonbook:\tRESET\n");

            rewind(pc_p->filehandle);

            pc_p->previous_operation = FIRSTMATCH;

            return(NORMAL);

            break;

        case  FIRSTMATCH:
            /*
            **  Look for first match of a string, starting at the
            **    beginning of the file...
            */
            printf("Phonbook:\tFIRSTMATCH\n");

            rewind(pc_p->filehandle);

            break;


        case  NEXTMATCH:
            /*
            **  Nothing special here, fall out and continue search
            */
            printf("Phonbook:\tNEXTMATCH\n");
            break;


        case  BROWSE:
            /*
            **  A BROWSE operation just returns the next entry...
            **
            **  If the last operation was a BROWSE that got an EOF,
            **     then rewind and start cycling through again.
            */
            printf("Phonbook:\tBROWSE\n");

            if (pc_p->previous_operation == BROWSE_RESET)
                rewind (pc_p->filehandle);
           
            if ((getfileline(buf,pc_p->filehandle)) != -1)
                {

                /*
                **  If not EOF, then just return next entry.
                */
                strcpy ((char *)stringout,(char *)buf);

                printf("Phonbook: \tFound %s\n", buf);

                pc_p->previous_operation = BROWSE;

                return(NORMAL);
                }
            else
                {
                /* 
                **  This allows the client to flag "no more entries"
                **  before cycling through the file again on 
                **  another BROWSE request.
                */ 
                pc_p->previous_operation = BROWSE_RESET;
        
                return(END);
                }
        }

    /*
    ** Keep track of previous operation in p_context
    */
    pc_p->previous_operation = op; 

    /*
    **  Either return the line of the file that contains a string
    **  match, or return -1...
    */

    while ((getfileline(buf,pc_p->filehandle)) != -1)
        {
        if ((strstr((char *)buf, (char *)stringin)) != (char *) NULL)
            {
            printf("Phonbook: \tFound %s\n", buf);
            strcpy ((char *)stringout,(char *)buf);
            return(NORMAL);
            }
        }

    return(END);
}
